// Importerer ndvendige pakker
import gl4java.*;			// For  kunne implementere GLEnum (gir tilgang til en rekke fields (variablene i store bokstaver))
import gl4java.utils.glut.*;	// Open GL's "verktyboks"

/**
 * <strong>Description:</strong><br>
 * Subclass of the abstract class Material, 
 * which supplies all necessary common methods for its subclasses.<br>
 * BlueGlassMaterial supplies methods that are particular to the component.<br>
 * Used for making a blue glass material for the different bulbs (and maybe stands) of the hourglass.<br>
 * <br><strong>All code is extensively commented in Norwegian.</strong>
 * @author Therese Rsholdt, Grafisk Databehandling 2003, Hi
 * @version 0.1
 */
public class BlueGlassMaterial extends Material
{
	/**
	* BlueGlassMaterial constructor
	* @param gl the GLFunc to pass on to the superclass Material
	*/
	public BlueGlassMaterial( GLFunc gl)
	{
		super(gl);
	}
	
	/**
	* Enables the BlueGlassMaterial
	* @param sideChoice the constant to indicate which side 
	* of the objekt should have this material, can be GL_FRONT_AND_BACK, 
	* GL_FRONT or GL_BACK.
	*/
	public void enableMaterial(int sideChoice)
	{       
		// Aktiviserer muligheten for  blande farger, slik at det kan vises gjennomsiktig
		gl.glEnable(GL_BLEND);
		// Setter depthbufferen i read only modus (slr den IKKE av!)
		gl.glDepthMask(GL_FALSE);
		// Stiller inn blendingsfunkjonen
		gl.glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		// Hvis det er materiale p begge sider skal det ogs reagere p lys p begge sider
		if (sideChoice == GL_FRONT_AND_BACK)
		{
			gl.glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,1);
		}
		
		// Bltt glass, setter riktig lyssetting for  f glasset gjennomsiktig og med riktig glans i.        
		float amb[] = {0.20f, 0.50f, 1.0f, 0.1f};
		float diff[] = {0.20f, 0.50f, 1.0f, 0.1f};
		float spec[] = {1.0f, 1.0f, 1.0f, 1.0f};
		float shine = 0.8f; // Mye glans i glasset
		
		gl.glMaterialfv(sideChoice, GL_AMBIENT, amb);
		gl.glMaterialfv(sideChoice, GL_DIFFUSE, diff);
		gl.glMaterialfv(sideChoice, GL_SPECULAR, spec);
		gl.glMaterialf(sideChoice, GL_SHININESS, shine * 128.0f);
	}
	
	/**
	* Restores default values
	*/
	public void disableMaterial()
	{       
		gl.glDisable(GL_BLEND); // Pasifiserer muligheten for  blande farger
		gl.glDepthMask(GL_TRUE); // Setter depthbufferen tilbake til read & write modus
		gl.glBlendFunc(GL_ONE, GL_ZERO); // Setter blendingsfunksjonen tilbake til defaultsettingen
		gl.glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, 0); // Setter lysmodellen tilbake til defaultsettingen
	}
}