// Importerer ndvendige pakker
import gl4java.*;			// For  kunne implementere GLEnum og GLUEnum (gir tilgang til en rekke fields (variablene i store bokstaver))
import gl4java.utils.glut.*;	// Open GL's "verktyboks"

/**
 * <strong>Description:</strong><br>
 * A "classic" stand, used together with <code>ClassicBulbs</code> to make an hourglass.<br>
 * <br><strong>All code is extensively commented in Norwegian.</strong>
 * @author Therese Rsholdt, Grafisk Databehandling 2003, Hi
 * @version 0.1
 */
public class ClassicStand extends Stand
{    	
	/**
	* ClassicStand constructor
	* @param gl the <code>GLFunc</code> interface that is used for this component
	* @param glu the <code>GLUFunc</code> interface that is used for this component
	* @param glut the <code>GLUTFunc</code> interface that is used for this component
	* @param bulbheight the height of the bulb, from the middle of the hourglass to the stand
	* @param endRadius the end radius of the bulb, used for setting the stands radius
	*/
	public ClassicStand ( GLFunc gl, GLUFunc glu, GLUTFunc glut, float bulbheight, float endRadius ) 
	{
		super(gl, glu, glut, bulbheight);
		standTypeName = "Classic";
		standRadius = endRadius * 1.5f;
	}
	
	/**
	* Draws a half hourglass stand
	* @param isLowerHalf used if the two halfs need to be different
	*/
	public void drawHalfStand(boolean isLowerHalf)
	{         
		// Setter radius for pinnene, og "smultringene"
		float pinRadius = 0.1f; 
		
		// Tegner de 4 pinnene med kuler sammen med den nederste delen
		if (isLowerHalf)
		{
			gl.glPushMatrix();
			gl.glRotatef(45.0f, 0.0f, 0.0f, 1.0f);
			
			for(int i=0; i<4; i++)
			{
				gl.glPushMatrix();
				gl.glRotatef(90.0f * i, 0.0f, 0.0f, 1.0f);
				gl.glTranslatef(standRadius - (pinRadius*2), 0.0f, 0.0f);
				glu.gluSphere(standPart, pinRadius*2, 40, 40);
				gl.glTranslatef(0.0f,0.0f, -innerHeight); // Pinnene blir tegnet med utganspunkt "i toppen"
				glu.gluCylinder(standPart, pinRadius, pinRadius, innerHeight * 2, 10, 1);
				gl.glPopMatrix();
			}

			gl.glPopMatrix();
		}
		
		// Setter en hyde p den "lille kanten" p sttten
		float standRimHeight = 0.02f;
		
		// Stttens bunn- og toppplater
		gl.glPushMatrix();
		gl.glTranslatef(0.0f, 0.0f, innerHeight);
		glu.gluQuadricOrientation(standPart, GLU_INSIDE); //M snu disken nrmest glasset for  f materiale p rett side
		glu.gluDisk(standPart, 0.0, standRadius, 80, 1);
		glu.gluQuadricOrientation(standPart, GLU_OUTSIDE); // Resten av sttten skal ha default innside/utside
		gl.glTranslatef(0.0f, 0.0f, pinRadius - standRimHeight);
		glut.glutSolidTorus(pinRadius, standRadius, 10, 80);
		gl.glTranslatef(0.0f, 0.0f, pinRadius);
		glut.glutSolidTorus(pinRadius, standRadius+pinRadius, 10, 80);
		gl.glTranslatef(0.0f, 0.0f, pinRadius - standRimHeight);
		glu.gluDisk(standPart, 0.0, standRadius+pinRadius, 80, 1);
		gl.glPopMatrix();
	}
}