//Importerer ndvendige pakker
import java.awt.*; // For  kunne bruke klassen Dimension
import java.awt.event.*; // For  kunne bruke MouseHandler
import gl4java.awt.GLAnimCanvas; // For at klassen skal kunne arve fra GLCanvas
import gl4java.utils.glut.*; // Open GL's "verktyboks"

/**
 * <strong>Description:</strong><br>
 * This is where all the fun takes part in making an Hourglass.<br>
 * <br><strong>All code is extensively commented in Norwegian.</strong>
 * @author Therese Rsholdt, Grafisk Databehandling 2003, Hi
 * @version 0.1
 */
public class HourglassAnimCanvas extends GLAnimCanvas
{
	GLUTFunc glut = null;
	MouseHandler mHandler;
	Bulbs bulbs;
	Stand stand;
	// Variabler som brukes til  kontrollere rotering  
	float rotx = 0.0f; 
	float rotz = 0.0f;   
	float lastx = 0.0f; 
	float lastz = 0.0f;
	
	// Variabler som kontrollerer om akser og kontrolpunkter skal vises
	boolean axesClicked = false;
	boolean ctrPointsClicked = false;
	
	// Variabel som kontrollerer om sanden skal vises eller ikke
	boolean wantSand = true;
	
	// Variabel for  sjekke om musen er i bruk
	boolean mouseMoving = false;
	    
	// VARIABLER FOR  LAGE SANDEN OG STYRE ANIMASJONEN
	// Generelle
	String bulbsTypeName = "";
	boolean turnButtonClicked = false;
	boolean isTurning = false;
	float tempRotZ = 0.0f;
	int frameCounter = 0;
	
	// For sanden i nederste del
	boolean needsBottomLowerSand = true; // M ha sand oppover glassets vegger p pningsbildet
	boolean trickle = false;
	float lowerT;
	float lowerTinyHeapRadius;
	int lowerTinyHeapCounter = 0; 
	int bottomLowerSandCounter = 0; 
	float lowerTinyHeapRadiusStartStep = 0.0f;
	float lowerTinyHeapRadiusGrowth = 0.0f;
	float lowerTinyHeapRadiusGrowthFactor = 0.0f;
	
	// For sanden i verste del
	boolean needsUpperSand = false;
	float upperT; 
	int upperSandCounter1 = 0;
	int upperSandCounter2 = 0;
	int upperSandCounter3 = 0;
	int upperSandCounter4 = 0;
	float upperSandShrink1 = 0.0f;
	float upperSandShrink2 = 0.0f;
	float upperSandShrink3 = 0.0f;
	float upperSandShrink4 = 0.0f;
	
	    
	/**
	* HourglassAnimCanvas constructor
	* @param w the width of this component
	* @param h the height of this component
	*/
	public HourglassAnimCanvas(int w, int h)
	{
		super(w, h);
		// Legger til mouseListener p canvasen (for  kunne rotere)
		mHandler = new MouseHandler();
		addMouseListener( mHandler );
		addMouseMotionListener( mHandler );
	}
	
	/**
     * Initialises the canvas before creating the GLContext.
     * <p>
     * Called just BEFORE the GL-Context is created.
     * <p>
     * Defines a reasonable set of 3D options of this component.  
     */
	public void preInit()
	{
		doubleBuffer = true;
		stereoView = false;
		accumSize = 8;
	}
	
	/**
     * Initialises the canvas after creating the GLContext.
     * <p>
     * Called just AFTER the GL-Context is created.   Set up the OpenGL state.
     * Sets the lighting, smooths the model and enables depthtesting
     */
	public void init()
	{
		glut = new GLUTFuncLightImpl(gl, glu);
		
		reshape(getSize().width, getSize().height);
		
		// Gjr klar lyskilden, helt "plain" lys
		float ambient[] = {0.6f,0.6f,0.6f,1.0f }; // Middels bakgrunnslys
		float diffuse[] = {1.0f,1.0f,1.0f,1.0f }; // "Fargen p lyset", sterkt hvitt lys
		float specular[] ={1.0f, 1.0f, 1.0f, 1.0f };
		float position[] = {1.0f,1.0f,1.0f,0.0f };
		
		gl.glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
		gl.glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
		gl.glLightfv(GL_LIGHT0, GL_POSITION, position);
		gl.glLightfv(GL_LIGHT0, GL_SPECULAR, specular);
		
		gl.glEnable(GL_LIGHT0);
		gl.glEnable(GL_LIGHTING);     
		
		// Aktiviserer glatting
		gl.glShadeModel(GL_SMOOTH);
		
		// Aktiviserer nomalisering
		gl.glEnable(GL_AUTO_NORMAL); 
		gl.glEnable(GL_NORMALIZE);
		
		// Aktiviserer dybdetesting (m vre aktivisert for  kunne bruke glDepthMask() under blendingen
		gl.glEnable(GL_DEPTH_TEST); // Defaultsettingen er gl.glDepthFunc(GL_LESS)
		
		// Setter bakgrunnsfarge:
		gl.glClearColor(1.0f, 1.0f, 1.0f, 1.0f);//Hvit
		
		// Instansierer glasset av nsket materiale, og med nsket sandmateriale
		// bulbs m initieres fr stand
		bulbs = (Bulbs) new ClassicBulbs(gl, glu, glut);
		bulbs.setBulbsMaterial("BlueGlass");
		bulbs.setSandMaterial("RedSand");
		
		// Instansierer "holder" av nsket materiale
		stand = (Stand) new ClassicStand(gl, glu, glut, bulbs.getBulbheight(), bulbs.getEndBulbRadius());
		stand.setStandMaterial("Brass");
		
		// Initierer variablene som bestemmer "fasongen" p sanden i nederste del
		// M initieres her for  f korrekt "pningsbilde"
		lowerT = bulbs.getMaxLowerSandLevel(); // ClassicBulbs: 0.1f
		
		// Initierer variablene som bestemmer "fasongen" p sanden i verste del
		upperT = bulbs.getMaxUpperSandLevel(); // ClassicBulbs: 0.6f
	}
		
	/**
	* Ensures GL is initialised correctly
	* Calls showScene() if GL4Java is initialized, and makes
	* the GLContext current for this thread
	*/
	public void display()
	{
		if (glj.gljMakeCurrent() == false)
			return;	
			
		if (turnButtonClicked)
		{
			frameCounter++;
			runAnimation();		
		}
		showScene();
		
		glj.gljSwap();
		glj.gljFree();
	}
	
	/**
	* Takes care of the animation
	*/
	public void runAnimation()
	{
		// Frste bilde i snuoperasjonen
		if(frameCounter == 1)
		{
			bulbsTypeName = bulbs.toString();
			isTurning = true; // Vil bare "symbolisere" sanden mens timeglasset snus
			tempRotZ = rotz; // Tar vare p Y-originalposisjon fr snuingen begynner
			rotz+=25.0f;
			
			lowerTinyHeapCounter++;
			bottomLowerSandCounter++;
			
			upperSandCounter1++;
			upperSandCounter2++;
			upperSandCounter3++;
			upperSandCounter4++;
		}
		
		// Mellomhakkene i snuoperasjonen, 25 grader om gangen
		if(frameCounter > 1 && frameCounter <= 6)
		{
			rotz+=25.0f;
			
			lowerTinyHeapCounter++;
			bottomLowerSandCounter++;
			
			upperSandCounter1++;
			upperSandCounter2++;
			upperSandCounter3++;
			upperSandCounter4++;
		}
		
		// Siste hakket i snuoperasjoen (timeglasset akkurat ferdig snudd)
		if(frameCounter == 7)
		{
			rotz = tempRotZ; // Skal rotere timeglasset til Y-originalposisjon
			isTurning = false; // N skal sanden tegnes "vanlig" igjen
			
			lowerTinyHeapCounter++;
			bottomLowerSandCounter++;
			needsBottomLowerSand = false; // Trenger ikke sand opp langs veggen i nedre halvdel enda
			trickle = true; // Trenger sandstrle fra n av
			lowerT = 0.0f; // "lowerTinyHeap'en" m ligge p bunnen av glasset, ingen Beziersand enda
			lowerTinyHeapRadius = 0.0f; // Ingen haug enda
			
			upperSandCounter1++;
			upperSandCounter2++;
			upperSandCounter3++;
			upperSandCounter4++;
			needsUpperSand = true;// Trenger sand i verste del av timeglasset
			upperT = bulbs.getMaxUpperSandLevel(); // ClassicBulbs 0.8f;        
		}
		
		// Herfra skal haugen ("lowerTinyHeap") i nederste del ke,
		// setter variabler for at den skal kunne ke suksessivt
		// ingen sand oppover veggen enda
		if(frameCounter == 8)
		{
			lowerTinyHeapCounter += bulbs.getLowerTinyHeapSteps();
			bottomLowerSandCounter++;
			lowerTinyHeapRadiusStartStep = 0.05f;
			lowerTinyHeapRadiusGrowthFactor = 1.0f; //faktor for king
			lowerTinyHeapRadius = lowerTinyHeapRadiusStartStep;
			
			upperSandCounter1 += bulbs.getUpperSandSteps1();
			upperSandCounter2++;
			upperSandCounter3++;
			upperSandCounter4++;
			upperSandShrink1 = bulbs.getUpperSandShrink1();
			upperSandShrink2 = bulbs.getUpperSandShrink2();
			upperSandShrink3 = bulbs.getUpperSandShrink3();
			upperSandShrink4 = bulbs.getUpperSandShrink4();
			upperT -= upperSandShrink1;
		}
		
		// Beziersandens stigning i den verste delen, n blir det forskjellig for hver glassfasong
		// Funksjoner av upperT styrer n bde klippeplanet og radien p "lokket" av Beziersanden
		if(frameCounter > 8 && frameCounter <= upperSandCounter1)
		{
			upperT -= upperSandShrink1;

			if(frameCounter == upperSandCounter1)
			{
				upperSandCounter2 += bulbs.getUpperSandSteps2() + 1;
			}
			else
			{
				upperSandCounter2++;
			}
			upperSandCounter3++;
			upperSandCounter4++;
		}
		
		if(frameCounter > upperSandCounter1 && frameCounter <= upperSandCounter2)
		{
			upperT -= upperSandShrink2;
			
			if(frameCounter == upperSandCounter2)
			{
				upperSandCounter3 += bulbs.getUpperSandSteps3() + 1;
			}
			else
			{
				upperSandCounter3++;
			}
			upperSandCounter4++;
		}
		
		if(frameCounter > upperSandCounter2 && frameCounter <= upperSandCounter3)
		{
			upperT -= upperSandShrink3;
			
			if(frameCounter == upperSandCounter3)
			{
				upperSandCounter4 += bulbs.getUpperSandSteps4() + 1;
			}
			else
			{
				upperSandCounter4++;
			}
		}
		
		if(frameCounter > upperSandCounter3 && frameCounter <= upperSandCounter4)
		{
			upperT -= upperSandShrink4;
			
			if(frameCounter == upperSandCounter4 - 3)
			{
				upperT = 0.32f;
			}
			
			if(frameCounter == upperSandCounter4 - 3)
			{
				upperT = 0.28f;
			}
			
			if(frameCounter == upperSandCounter4 - 2)
			{
				upperT = 0.22f;
			}
			
			if(frameCounter == upperSandCounter4 - 1)
			{
				upperT = 0.14f;
			}
		}
		
		// Her ker haugen i nederste del suksessivt til steget fr den treffer Beziersanden
		// lowerTinyHeapRadius m vokse i mindre og mindre steg
		if(frameCounter > 8 && frameCounter <= lowerTinyHeapCounter)
		{
			bottomLowerSandCounter = frameCounter + bulbs.getBottomLowerSandSteps();
			lowerTinyHeapRadiusGrowth = lowerTinyHeapRadiusStartStep * lowerTinyHeapRadiusGrowthFactor;
			lowerTinyHeapRadius += lowerTinyHeapRadiusGrowth;
			lowerTinyHeapRadiusGrowthFactor -= 0.0175f;
		}
		
		// "lowerTinyHeap" treffer Beziersanden i nederste del
		if(frameCounter == lowerTinyHeapCounter + 1)
		{
			needsBottomLowerSand = true; // Trenger sand opp langs veggen fra n av
			lowerTinyHeapRadius = bulbs.getEndSandRadius(); //Siste gang lowerTinyHeapRadius blir satt
		}
		
		// Beziersandens stigning i den nederste delen, n blir det forskjellig for hver glassfasong
		// Funksjoner av lowerT styrer n bde klippeplanet og radien p toppen av Beziersanden
		if(frameCounter > lowerTinyHeapCounter && frameCounter <= bottomLowerSandCounter)
		{
			// Hvis fasongen er klassisk eller moderne slipper jeg "unna" med en fast kning
			if(bulbsTypeName.equals("Classic") || bulbsTypeName.equals("Modern"))
			{
				lowerT += 0.0012f;
			}
		}
		
		// Trenger ikke sand i verste del av timeglasset, eller sandstrle, lenger
		if(frameCounter > bottomLowerSandCounter)
		{
			frameCounter = 0;
			lowerTinyHeapCounter = 0;
			bottomLowerSandCounter = 0;
			lowerT = bulbs.getMaxLowerSandLevel();
			trickle = false;
			
			upperSandCounter1 = 0;
			upperSandCounter2 = 0;
			upperSandCounter3 = 0;
			needsUpperSand = false;// Trenger ikke sand i verste del av timeglasset, eller sandstrle, lenger
			turnButtonClicked = false;
			stop();
		}
		
		//System.out.println("HOURGLASS:                                            Fra runAnimation(), frameCounter: " + frameCounter);
		//System.out.println("HOURGLASS:         Fra runAnimation(), lowerTinyHeapRadiusGrowth: " + lowerTinyHeapRadiusGrowth);
		//System.out.println("HOURGLASS:         Fra runAnimation(), lowerTinyHeapRadius: " + lowerTinyHeapRadius);
		//System.out.println("HOURGLASScanvas:          Fra runAnimation(), lowerTinyHeapCounter: " + lowerTinyHeapCounter);
		//System.out.println("HOURGLASScanvas:     Fra runAnimation(), bottomLowerSandCounter: " + bottomLowerSandCounter);
		//System.out.println("                                                                                   UPPER T: " + upperT);
		//System.out.println("HOURGLASScanvas:               Fra runAnimation(), upperSandCounter1: " + upperSandCounter1);
		//System.out.println("HOURGLASScanvas:               Fra runAnimation(), upperSandCounter2: " + upperSandCounter2);
		//System.out.println("HOURGLASScanvas:               Fra runAnimation(), upperSandCounter3: " + upperSandCounter3);
		//System.out.println("HOURGLASScanvas:               Fra runAnimation(), upperSandCounter4: " + upperSandCounter4);
	}
		
	/**
	* Clears the necessary buffers, sets the viewing transformations
	* and calls for drawing of the hourglass
	*/
	void showScene()
	{
		gl.glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		gl.glClear(GL_ACCUM_BUFFER_BIT);
		gl.glLoadIdentity();
		
		glu.gluLookAt(0.0f,0.0f,12.0f, // avstanden til "yet"
						0.0f,0.0f,0.0f,	// Ser mot origo
						0.0f,1.0f,0.0f); // Velger at Y-aksen skal vre "opp" (fr det roteres p plass)
		
		// Roterer rundt X-, Y- og Z-aksen
		// Gjres ikke dette kan ikke musebruk f noen effekt, og glasset heller ikke snus
		gl.glRotatef(rotx,1.0f,0.0f,0.0f);
		// Roterer rundt y-aksen  for  se timeglaset i riktig vinkel "fra siden"
		gl.glRotatef(12.0f,0.0f,1.0f,0.0f);
		// M rotere runt Z-aksen da timeglasset skal snus under animasjonen
		gl.glRotatef(rotz,0.0f,0.0f,1.0f);
		
		// Testmetode som setter inn kuler, for  teste gjennomsiktigheten under utviklingen av programmet
		//testAddSpheres();
		
		// Kaller tegning av timeglasset		
		drawHourglass();
		
		//Rydder opp.
		gl.glFlush();        
	}
	
	/**
	* Calls the necessary methods for the bulbs, stand and sand to be drawn.
	*/
	public void drawHourglass()
	{
		// ALT SOM SKAL SEES GJENNOM GLASSET M TEGNES FRST
		// Tegner aksene hvis bruker har bedt om det
		if (axesClicked)
		{
			drawAxes();
		}
		
		// Tegner linjer mellom kontrollpunktene hvis bruker har bedt om det
		if (ctrPointsClicked)
		{		
			bulbs.drawBulbCtrPointLines();
		}
		
		// Tegner sttten
		stand.drawStand();
		
		if(bulbs.toString().equals("Classic") || bulbs.toString().equals("Modern"))
		{
			// Tegner sanden hvis brukeren vil (og det er "lovlig")
			if (wantSand)
			{
				// Aktiviserer sandens materiale herfra for  slippe mange kall i Bulbs
				bulbs.activateSandMaterial();
				
				// Hvis timeglasset ikke er i snufasen skal sanden tegnes "ordentlig"
				if (!isTurning)
				{
					// Glasset trenger sand i nederste del
					bulbs.drawLowerSand(needsBottomLowerSand, trickle, lowerT, lowerTinyHeapRadius);
					
					// Hvis glasset trenger sand i verste del
					if (needsUpperSand)
					{
						bulbs.drawUpperSand(upperT);
					}
				}
				// Hvis timeglasset er i snufasen "simuleres" sanden
				else
				{
					bulbs.drawSandWhileTurning();
				}
				// Pasifiserer sanden materiale herfra for  slippe mange kall i Bulbs
				bulbs.deactivateSandMaterial();
			}
		}
		
		// Tegner glasset
		bulbs.drawBulbs();
	}
	
	/**
	* Puts the hourglass back up to its original position
	*/
	public void putHourglassBackUp()
	{
		rotx = 00.0f; 
		rotz = 0.0f;   
		lastx = 0.0f; 
		lastz = 0.0f;
		display();
	}
	
	/**
	* Gets called when the size of the canvas changes
	* @param w the width of this component
	* @param h the height of this component
	*/
	public void reshape(int w, int h)
	{
		gl.glViewport(0, 0, w, h);
		gl.glMatrixMode(GL_PROJECTION);
		gl.glLoadIdentity();
		glu.gluPerspective( 45.0, (float) w/h, 1.0, 500.0 );
		gl.glMatrixMode(GL_MODELVIEW);
	}
	
	/**
	* Changes the type of bulbs used for the hourglass.<br>
	* If the stand is made of <code>BrassMaterial</code> or <code>EbonyMaterial</code>
	* the stand alse changes to its "standard" type.
	* @param type the type of bulbs to make
	*/
	public void changeBulbsType(String type)
	{
		if(!turnButtonClicked)
		{
			String tempBulbsMaterial = bulbs.getBulbsMaterial();
			String tempSandMaterial = bulbs.getSandMaterial();
			String tempStandMaterial = stand.getStandMaterial();
			
			if (type.equals("Klassisk"))
			{
				bulbs = (Bulbs) new ClassicBulbs(gl, glu, glut);
				if (tempStandMaterial.equals("Brass") ||  tempStandMaterial.equals("Ebony"))
				{
					stand = (Stand) new ClassicStand(gl, glu, glut, bulbs.getBulbheight(), bulbs.getEndBulbRadius());
				}
			}
			
			if (type.equals("Moderne"))
			{
				bulbs = (Bulbs) new ModernBulbs(gl, glu, glut);
				if (tempStandMaterial.equals("Brass") ||  tempStandMaterial.equals("Ebony"))
				{
					stand = (Stand) new ModernStand(gl, glu, glut, bulbs.getBulbheight(), bulbs.getEndBulbRadius());
				}
			}
			
			if (type.equals("Bisarr"))
			{
				bulbs = (Bulbs) new BizarreBulbs(gl, glu, glut);
				if (tempStandMaterial.equals("Brass") ||  tempStandMaterial.equals("Ebony"))
				{
					stand = (Stand) new BizarreStand(gl, glu, glut, bulbs.getBulbheight(), bulbs.getEndBulbRadius());
				}
			}
			
			if (stand.toString().equals("Simple"))
			{
				stand = (Stand) new SimpleStand(gl, glu, glut, bulbs.getBulbheight(), bulbs.getEndBulbRadius());
			}
			
			if (stand.toString().equals("Plain"))
			{
				stand = (Stand) new PlainStand(gl, glu, glut, bulbs.getBulbheight(), bulbs.getEndBulbRadius());
			}
		
			bulbs.setBulbsMaterial(tempBulbsMaterial);
			bulbs.setSandMaterial(tempSandMaterial);
			stand.setStandMaterial(tempStandMaterial);
		}
		display();
	}
	
	/**
	* Changes the type of stand used for the hourglass.<br>
	* If the stand is made of <code>BrassMaterial</code> or <code>EbonyMaterial</code>
	* the stand also changes to its "standard" type.
	* @param type the type of stand to make
	*/
	public void changeStandType(String type)
	{
		if (type.equals("Brass") || type.equals("Ebony"))
		{
			if(bulbs.toString().equals("Classic"))
			{
				stand = (Stand) new ClassicStand(gl, glu, glut, bulbs.getBulbheight(), bulbs.getEndBulbRadius());
			}
			if(bulbs.toString().equals("Modern"))
			{
				stand = (Stand) new ModernStand(gl, glu, glut, bulbs.getBulbheight(), bulbs.getEndBulbRadius());
			}
			if(bulbs.toString().equals("Bizarre"))
			{
				stand = (Stand) new BizarreStand(gl, glu, glut, bulbs.getBulbheight(), bulbs.getEndBulbRadius());
			}
			stand.setStandMaterial(type);
		}
		
		if (type.equals("Enkel"))
		{
			stand = (Stand) new SimpleStand(gl, glu, glut, bulbs.getBulbheight(), bulbs.getEndBulbRadius());
			stand.setStandMaterial("Green");
		}
		
		if (type.equals("Glassbunn"))
		{
			stand = (Stand) new PlainStand(gl, glu, glut, bulbs.getBulbheight(), bulbs.getEndBulbRadius());
			stand.setStandMaterial(bulbs.getBulbsMaterial()); // Glassbunnen skal ha samme materiale som glasset
		}
		display();
	}
	
	/**
	* Changes the material of the bulbs used for the hourglass.
	* @param material the kind of material to make
	*/
	public void changeBulbsMaterial(String material)
	{
		bulbs.setBulbsMaterial(material);
		if (stand.toString().equals("Plain"))
		{
			stand.setStandMaterial(material);
		}
		display();		
	}
	
	/**
	* Changes the material of the sand used in the hourglass.
	* @param material the kind of material to make
	*/
	public void changeSandMaterial(String material)
	{
		bulbs.setSandMaterial(material);
		display();
	}
			
	public void setTurnButtonClicked()
	{
		turnButtonClicked = true;
		start();
	}
	
	public void setAxesClicked(ActionEvent e)
	{
		axesClicked =!axesClicked;
		display();
	}
	
	public void setCtrPointsClicked(ActionEvent e)
	{
		ctrPointsClicked =!ctrPointsClicked;
		display();
	}
	
	public void setWantSandClicked(ActionEvent e)
	{
		wantSand =!wantSand;
		display();
	}
	
	
	/**
	* Inner class for mouse events
	*/
	class MouseHandler implements MouseListener, MouseMotionListener
	{
		public void mouseEntered(MouseEvent e) {}
		public void mouseExited(MouseEvent e) {}
		public void mouseMoved(MouseEvent e) {}
		public void mouseClicked(MouseEvent e){}
		public void mouseReleased(MouseEvent e)
		{
			// objektet beveger seg ikke lenger
			mouseMoving=false;
			repaint();
		}
		public void mousePressed(MouseEvent e)
		{ 
			lastx = e.getX();
			lastz = e.getY();
			mouseMoving=true;
		}

		/*************************************
		*Roteringskode, lnt av Brres hitbox.
		**************************************/
		public void mouseDragged(MouseEvent e)
		{
			if(!mouseMoving)
				return;
			
			int x = e.getX();
			int y = e.getY();
			
			Dimension dim = e.getComponent().getSize();
			int width = dim.width;
			int height = dim.height;
			
			rotz += (90.0f*((float)(x-lastx)/(float)(width)));
			rotx += (90.0f*((float)(y-lastz)/(float)(height)));
			//roty += (90.0f*((float)(y-lastz)/(float)(height)));//////////////
			lastx = x;
			lastz = y;
			
			// repaint with moving flag set
			repaint();
		}
	}
	
	/*Tegner ut de positive aksene:
	X = rd, Y = grnn, Z = bl      */
	public void drawAxes()
	{
		gl.glLineWidth(3);
		
		// Rd farge til X
		float ambR[]={1.0f, 0.0f, 0.0f, 1.0f};
		float diffR[]={1.0f, 0.0f, 0.0f, 1.0f};
		float specR[]={0.0f, 0.0f, 0.0f, 1.0f};
		float shineR=32.00f;
		gl.glMaterialfv(GL_FRONT,GL_AMBIENT,ambR);
		gl.glMaterialfv(GL_FRONT,GL_DIFFUSE,diffR);
		gl.glMaterialfv(GL_FRONT,GL_SPECULAR,specR);
		gl.glMaterialf(GL_FRONT,GL_SHININESS,shineR);
		// Tegner +X
		gl.glBegin(GL_LINES);
		gl.glVertex3f(0.0f, 0.0f, 0.0f); 
		gl.glVertex3f(4.0f, 0.0f, 0.0f);
		gl.glEnd();
				
		// Grnn farge til Y
		float ambG[]={0.0f, 1.0f, 0.0f, 1.0f};
		float diffG[]={0.0f, 1.0f, 0.0f, 1.0f};
		float specG[]={0.0f, 0.0f, 0.0f, 1.0f};
		float shineG=32.00f;
		gl.glMaterialfv(GL_FRONT,GL_AMBIENT,ambG);
		gl.glMaterialfv(GL_FRONT,GL_DIFFUSE,diffG);
		gl.glMaterialfv(GL_FRONT,GL_SPECULAR,specG);
		gl.glMaterialf(GL_FRONT,GL_SHININESS,shineG);
		// Tegner +Y
		gl.glBegin(GL_LINES);
		gl.glVertex3f(0.0f, 0.0f, 0.0f); 
		gl.glVertex3f(0.0f, 4.0f, 0.0f);
		gl.glEnd();
				
		// Bl farge til Z
		float ambB[]={0.0f, 0.0f, 1.0f, 1.0f};
		float diffB[]={0.0f, 0.0f, 1.0f, 1.0f};
		float specB[]={0.0f, 0.0f, 0.0f, 1.0f};
		float shineB=32.00f;
		gl.glMaterialfv(GL_FRONT,GL_AMBIENT,ambB);
		gl.glMaterialfv(GL_FRONT,GL_DIFFUSE,diffB);
		gl.glMaterialfv(GL_FRONT,GL_SPECULAR,specB);
		gl.glMaterialf(GL_FRONT,GL_SHININESS,shineB);
		// Tegner +Z
		gl.glBegin(GL_LINES);
		gl.glVertex3f(0.0f, 0.0f, 0.0f); 
		gl.glVertex3f(0.0f, 0.0f, 4.0f);
		gl.glEnd();
		
		gl.glLineWidth(1);
	}
	
	/**
	* Test method
	*/
	public void testAddSpheres()
	{
		// Setter inn noen kuler for  teste gjennomsiktigheten
		// Gullkule i verste halvdel av glasset
		float amb1[]={0.24725f,0.1995f,0.0745f,1.0f};
		float diff1[]={0.75164f,0.60648f,0.22648f,1.0f};
		float spec1[]={0.628281f,0.555802f,0.366065f,1.0f};
		float shine1=32.00f;
		gl.glMaterialfv(GL_FRONT,GL_AMBIENT,amb1);
		gl.glMaterialfv(GL_FRONT,GL_DIFFUSE,diff1);
		gl.glMaterialfv(GL_FRONT,GL_SPECULAR,spec1);
		gl.glMaterialf(GL_FRONT,GL_SHININESS,shine1);
		
		gl.glPushMatrix();
		gl.glTranslatef(0.0f, 2.0f, 0.0f);
		glut.glutSolidSphere (0.6f, 20, 20);
		gl.glPopMatrix();
		
		// Slvkule i nederste del av glasset
		float amb[]={0.19225f, 0.19225f, 0.19225f, 1.0f};
		float diff[]={0.50754f, 0.50754f, 0.50754f, 1.0f};
		float spec[]={0.508273f, 0.508273f, 0.508273f, 1.0f};
		float shine=32.00f;
		gl.glMaterialfv(GL_FRONT,GL_AMBIENT,amb);
		gl.glMaterialfv(GL_FRONT,GL_DIFFUSE,diff);
		gl.glMaterialfv(GL_FRONT,GL_SPECULAR,spec);
		gl.glMaterialf(GL_FRONT,GL_SHININESS,shine);
		
		gl.glPushMatrix();
		gl.glTranslatef(0.0f,-2.0f,0.0f);
		glut.glutSolidSphere (0.6f, 20, 20);
		gl.glPopMatrix();
		
		// Messingkule p utsiden av glasset
		float amb2[]={0.399412f, 0.223529f, 0.027451f, 1.0f};
		float diff2[]={0.780392f, 0.568627f, 0.113725f, 1.0f};
		float spec2[]={0.992157f, 0.941176f, 0.807843f, 1.0f};
		float shine2=27.8974f;
		gl.glMaterialfv(GL_FRONT, GL_AMBIENT,amb2);
		gl.glMaterialfv(GL_FRONT, GL_DIFFUSE,diff2);
		gl.glMaterialfv(GL_FRONT, GL_SPECULAR,spec2);
		gl.glMaterialf(GL_FRONT, GL_SHININESS,shine2);
		
		gl.glPushMatrix();
		gl.glTranslatef(0.0f,0.0f,-2.0f);
		glut.glutSolidSphere (0.6f, 20, 20);
		gl.glPopMatrix();
	}             
}