//Importerer ndvendige pakker
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import gl4java.utils.glut.*; // Open GL's "verktyboks"

/**
 * <strong>Description:</strong><br>
 * The main class for making an Hourglass.<br>
 * Supplies the Frame and Main-method.<br>
 * <br><strong>All code is extensively commented in Norwegian.</strong>
 * @author Therese Rsholdt, Grafisk Databehandling 2003, Hi
 * @version 0.1
 */
public class HourglassFrame extends JFrame implements ActionListener
{
	JPanel contentPane;  
	WindowHandler wHandler; 
	HourglassAnimCanvas hourglassCanvas = null;   
	
	/**
	* HourglassFrame constructor
	*/     
	public HourglassFrame()
	{
		//Bygger opp framen
		contentPane = (JPanel) this.getContentPane();
		contentPane.setLayout(new BorderLayout());
		this.setBounds(100, 50, 500, 500);
		this.setTitle("...Timeglass...");    
		
		Button putBackUp = new Button( "Originalposisjon" );
		putBackUp.addActionListener(this);
		contentPane.add("North", putBackUp);
		
		//Deler opp framen, med knapp p topp og canvas i midten.
		Dimension d = getSize();
		hourglassCanvas = new HourglassAnimCanvas(d.width, d.height);
		contentPane.add("Center", hourglassCanvas);
		
		Button turn = new Button( "Snu timeglass" );
		turn.addActionListener(this);
		contentPane.add("South", turn);
		
		// Setter opp menyene
		MenuBar bar = new MenuBar();
		
		Menu fileMenu = new Menu("Fil");
		String fileNames[] = {"Akser P/Av", "Kontrollpunkter P/Av", "Sand P/Av"};
		MenuItem fileItems[];
		fileItems = new MenuItem[fileNames.length];
		for (int i=0; i < fileItems.length; i++)
		{
			fileItems[i] =  new MenuItem(fileNames[i]);
			fileMenu.add(fileItems[i]);
			fileItems[i].addActionListener(this);
		}           
		bar.add(fileMenu);
		
		
		Menu bulbsShapeMenu = new Menu("Glassfasonger");
		String bulbsShapeNames[] = {"Klassisk", "Moderne", "Bisarr"};
		MenuItem bulbsShapeItems[];
		bulbsShapeItems = new MenuItem[bulbsShapeNames.length];
		for (int i=0; i < bulbsShapeItems.length; i++)
		{
			bulbsShapeItems[i] =  new MenuItem(bulbsShapeNames[i]);
			bulbsShapeMenu.add(bulbsShapeItems[i]);
			bulbsShapeItems[i].addActionListener(this);
		}           
		bar.add(bulbsShapeMenu);
		
		
		Menu bulbsMaterialMenu = new Menu("Glassfarger");
		String bulbsMaterialNames[] = {"Bltt", "Grnt", "Sotet"};
		MenuItem bulbsMaterialItems[];
		bulbsMaterialItems = new MenuItem[bulbsMaterialNames.length];
		for (int i=0; i < bulbsMaterialItems.length; i++)
		{
			bulbsMaterialItems[i] =  new MenuItem(bulbsMaterialNames[i]);
			bulbsMaterialMenu.add(bulbsMaterialItems[i]);
			bulbsMaterialItems[i].addActionListener(this);
		}           
		bar.add(bulbsMaterialMenu);
		
		Menu standTypeMenu = new Menu("Stttetyper");
		String standTypeNames[] = {"Messing", "Ibenholt", "Enkel", "Glassbunn"};
		MenuItem standTypeItems[];
		standTypeItems = new MenuItem[standTypeNames.length];
		for (int i=0; i < standTypeItems.length; i++)
		{
			standTypeItems[i] =  new MenuItem(standTypeNames[i]);
			standTypeMenu.add(standTypeItems[i]);
			standTypeItems[i].addActionListener(this);
		}           
		bar.add(standTypeMenu);
		
		
		Menu sandMaterialMenu = new Menu("Sandfarger");
		String sandMaterialNames[] = {"Rd", "Gul"};
		MenuItem sandMaterialItems[];
		sandMaterialItems = new MenuItem[sandMaterialNames.length];
		for (int i=0; i < sandMaterialItems.length; i++)
		{
			sandMaterialItems[i] =  new MenuItem(sandMaterialNames[i]);
			sandMaterialMenu.add(sandMaterialItems[i]);
			sandMaterialItems[i].addActionListener(this);
		}           
		bar.add(sandMaterialMenu);
		
		setMenuBar(bar);
		
		//legger til windowListener til frame
		wHandler=new WindowHandler();
		this.addWindowListener(wHandler);   
		
		pack();
		show();
	}

	/**
	* Main method
	*/
	public static void main(String[] args)
	{
		new HourglassFrame();
	} 
	
	/**
	* Responds to the commands of the user
	* @param e the ActionEvent to occur
	*/    
	public void actionPerformed( ActionEvent e ) 
	{      
		// Sjekker hva som er trykket p
		String cmd = e.getActionCommand();  
		
		
		// FIL
		// Brukeren har bedt om  f de positive aksene tegnet ut, eller fjernet
		if (cmd.equals("Akser P/Av")) 
		{
			hourglassCanvas.setAxesClicked(e);
		}
		// Brukeren har bedt om  f de kontrolpunktene tegnet ut, eller fjernet
		if (cmd.equals("Kontrollpunkter P/Av")) 
		{
			hourglassCanvas.setCtrPointsClicked(e);
		}
		// Brukeren har bedt om at sanden skal tegnes, eller ikke
		if (cmd.equals("Sand P/Av")) 
		{
			hourglassCanvas.setWantSandClicked(e);
		}
		
		// GLASSFASONGER		
		// Brukeren har bedt om en klassisk fasong
		if (cmd.equals("Klassisk")) 
		{
			hourglassCanvas.changeBulbsType("Klassisk");
		}
		// Brukeren har bedt om en moderne fasong
		if (cmd.equals("Moderne")) 
		{
			hourglassCanvas.changeBulbsType("Moderne");
		}
		// Brukeren har bedt om en bisarr fasong
		if (cmd.equals("Bisarr")) 
		{
			hourglassCanvas.changeBulbsType("Bisarr");
		}
		
		
		// GLASSFARGER
		// Brukeren har bedt om bltt glass
		if (cmd.equals("Bltt")) 
		{
			hourglassCanvas.changeBulbsMaterial("BlueGlass");
		}
		// Brukeren har bedt om grnt glass
		if (cmd.equals("Grnt")) 
		{
			hourglassCanvas.changeBulbsMaterial("GreenGlass");
		}
		// Brukeren har bedt om sotet glass
		if (cmd.equals("Sotet")) 
		{
			hourglassCanvas.changeBulbsMaterial("SootyGlass");
		}
		
		
		// STTTETYPER
		// Brukeren har bedt om en sttte ("standard" type for sandbeholderen) i messing 
		if (cmd.equals("Messing")) 
		{
			hourglassCanvas.changeStandType("Brass");
		}
		// Brukeren har bedt om en sttte ("standard" type for sandbeholderen) i ibenholt 
		if (cmd.equals("Ibenholt")) 
		{
			hourglassCanvas.changeStandType("Ebony");
		}
		// Brukeren har bedt om en enkel sttte   
		if (cmd.equals("Enkel")) 
		{
			//System.out.println("\nHar trykket p Enkel");
			hourglassCanvas.changeStandType("Enkel");
		} 
		// Brukeren har bedt om at timeglasset bare skal ha glassbunn  
		if (cmd.equals("Glassbunn")) 
		{
			hourglassCanvas.changeStandType("Glassbunn");
		}
		
		
		// SANDFARGER
		// Brukeren har bedt om rd sand
		if (cmd.equals("Rd")) 
		{
			hourglassCanvas.changeSandMaterial("RedSand");
		}
		// Brukeren har bedt om rd sand
		if (cmd.equals("Gul")) 
		{
			hourglassCanvas.changeSandMaterial("YellowSand");
		}
		
		// Brukeren har bedt om  snu timeglasset til originalposisjonen   
		if (cmd.equals("Originalposisjon")) 
		{
			hourglassCanvas.putHourglassBackUp();
		} 
		
		// Brukeren har bedt om  snu timeglasset ved  klikke p knappen "Snu timeglass"   
		if (cmd.equals("Snu timeglass")) 
		{
			//hourglassCanvas.turnHourglass();
			hourglassCanvas.setTurnButtonClicked();
		} 
	}

	/**
	* Inner class for window events
	* @see WindowListener
	*/
	class WindowHandler implements WindowListener
	{
		public void windowOpened(WindowEvent parm1) {}
		public void windowClosing(WindowEvent parm1)
		{
			System.exit( 0 );
		}
		public void windowClosed(WindowEvent parm1) {}
		public void windowIconified(WindowEvent parm1) {}
		public void windowDeiconified(WindowEvent parm1) {}
		public void windowActivated(WindowEvent parm1) {}
		public void windowDeactivated(WindowEvent parm1) {}
	}
}